<?php
/**
 * Basic super-class for all Node objects
 * 
 * @author BrendonCrawford
 * @created 2009-11-08
 * @see http://drupal.org/project/orm
 *
 */
class OrmNode extends stdClass {
  
  /**
   * @constructor
   * @param $node Object[DrupalNode]
   * @param $opts Assoc
   */
  public function __construct($node, $opts=array()) {
    $methods = get_class_methods($this);
    foreach($node as $prop => $val) {
      if (in_array($prop, $methods)) {
        $prop = '_' . $prop;
      }
      $this->{$prop} = $val;
    }
  }

  /**
   * Saves a node
   * 
   * @return unknown_type
   */
  public function save($opts=array()) {
    $opts = array_merge(array(
      'recursive' => TRUE
    ), $opts);
    if ($opts['recursive']) {
      $this->save(array('recursive' => FALSE));
      $this->saveRecursive($this);
    }
    else {
      node_save($this);
    }
  }

  /**
   * Save recursively
   * 
   * @param $item
   * @return Bool
   */
  private function saveRecursive(&$item) {
    $c = __CLASS__;
    $m = __FUNCTION__;
    $is_obj = is_object($item);
    $is_arr = is_array($item);
    if ($is_obj || $is_arr) {
      foreach ($item as $prop => $val) {
        if ($is_obj) {
          $o =& $item->{$prop};
        }
        elseif ($is_arr) {
          $o =& $item[$prop];
        }
        // An OrmNode
        if ($o instanceof $c) {
          $o->save();
        }
        // An OrmNodeCollection
        elseif ($o instanceof OrmNodeCollection) {
          $o->saveAll();
        }
        else {
          $this->{$m}($o);
        }
      }
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * Delete
   * 
   * @return Bool
   */
  public function delete() {
    node_delete($this->nid);
    unset($this);
    return TRUE;
  }

  /**
   * Sets title
   * 
   * @param $val String
   * @return Bool
   */
  public function setTitle($val) {
    if (OrmUtil::dver(7)) {
      $title =
        array(FIELD_LANGUAGE_NONE => array(array('value' => $val)));
    }
    elseif (OrmUtil::dver(6)) {
      $title = $val;
    }
    $this->title = $val;
    return TRUE;
  }

}